#include <Wire.h>
#include <LiquidCrystal_I2C.h>

// Простой вольт-ампер-метр с i2c дисплеем 1602. V 1611

// Настройки i2c дисплея 1602 с нестандартной распиновкой
#define LCD_I2C_ADDR    0x27 
#define BACKLIGHT     3      
#define LCD_EN  2            
#define LCD_RW  1            
#define LCD_RS  0            
#define LCD_D4  4            
#define LCD_D5  5            
#define LCD_D6  6            
#define LCD_D7  7            
 
LiquidCrystal_I2C lcd(LCD_I2C_ADDR,LCD_EN,LCD_RW,LCD_RS,LCD_D4,LCD_D5,LCD_D6,LCD_D7);

// Время обновления показаний, мс (200 - 2000)
#define REFRESH_TIME  330

// Аналоговый вход
#define PIN_VOLT A0
#define PIN_AMP A1
// Внутреннее опорное напряжение (подобрать)
const float VRef = 1.10;
// Коэффициент входного резистивного делителя (Rh + Rl) / Rl. IN <-[ Rh ]--(analogInPin)--[ Rl ]--|GND
const float VoltMult = (180.0 + 10.0) / 10.0;
// Усиление по току
const float AmpMult = 1.0;
// Сопротивление шунта в омах (подобать)
const float AmpRx = 0.098;

void setup() {
  analogReference(INTERNAL);
  // Инициализация дисплея 
  lcd.begin (16, 2);
  lcd.setBacklightPin(BACKLIGHT, POSITIVE);
  lcd.setBacklight(HIGH); // включить подсветку
  lcd.clear(); // очистить дисплей
  lcd.print("VAMetr");
}

void loop() {
  unsigned long CalcStart = millis();
  float InVolt = 0;
  float InAmp = 0;
  int ReadCnt = 0;
  // Чтение из порта с усреднением
  while ((millis() - CalcStart) < REFRESH_TIME) {
    InVolt += analogRead(PIN_VOLT);
    InAmp += analogRead(PIN_AMP);
    ReadCnt++;
  }
  InVolt = InVolt / ReadCnt;
  InAmp = InAmp / ReadCnt;
  // Компенсация смещения 0 АЦП (подобрать)
  if (InVolt > 0.2) InVolt += 3;
  // Перевод в вольты (In: 0..1023 -> (0..VRef) scaled by Mult)
  float Volt = InVolt * VoltMult * VRef / 1023;
  float Amp = InAmp * VRef / AmpMult / AmpRx / 1023 ;
  // Для учета падения на шунте раскомментировать 2 строки
  //float RxVolt = InAmp * VRef / 1023 / AmpMult;
  // Volt -= RxVolt;
  float Watt = Volt * Amp;
  // Вывод данных
  lcd.setCursor (8, 0);
  lcd.print(Watt);
  lcd.print("W ");
  lcd.setCursor (0, 1);
  lcd.print(Volt);
  lcd.print("V ");
  lcd.setCursor (8, 1);
  lcd.print(Amp);
  lcd.print("A ");
}

